<?php

/**
 * Authenticate using LiveID.
 *
 * @author Brook Schofield, TERENA.
 * @package simpleSAMLphp
 * @version $Id$
 */
class sspmod_authwindowslive_Auth_Source_LiveID extends SimpleSAML_Auth_Source {

	/**
	 * The string used to identify our states.
	 */
	const STAGE_INIT = 'authwindowslive:init';

	/**
	 * The key of the AuthId field in the state.
	 */
	const AUTHID = 'authwindowslive:AuthId';

	private $key;
	private $secret;


	/**
	 * Constructor for this authentication source.
	 *
	 * @param array $info  Information about this authentication source.
	 * @param array $config  Configuration.
	 */
	public function __construct($info, $config) {
		assert('is_array($info)');
		assert('is_array($config)');

		/* Call the parent constructor first, as required by the interface. */
		parent::__construct($info, $config);

		if (!array_key_exists('key', $config))
			throw new Exception('LiveID authentication source is not properly configured: missing [key]');

		$this->key = $config['key'];

		if (!array_key_exists('secret', $config))
			throw new Exception('LiveID authentication source is not properly configured: missing [secret]');

		$this->secret = $config['secret'];
	}


	/**
	 * Log-in using LiveID platform
	 *
	 * @param array &$state  Information about the current authentication.
	 */
	public function authenticate(&$state) {
		assert('is_array($state)');

		/* We are going to need the authId in order to retrieve this authentication source later. */
		$state[self::AUTHID] = $this->authId;

		$stateID = SimpleSAML_Auth_State::saveState($state, self::STAGE_INIT);

		SimpleSAML_Logger::debug('authwindowslive auth state id = ' . $stateID);

		// Authenticate the user
		// Documentation at: http://msdn.microsoft.com/en-us/library/live/hh243649.aspx
		  $authorizeURL = 'https://login.live.com/oauth20_authorize.srf'
                               . '?client_id=' . $this->key
                                . '&scope=wl.basic'
                                . '&response_type=code'
                               . '&redirect_uri=' . urlencode(SimpleSAML_Module::getModuleUrl('authwindowslive') . '/linkback.php?auth_state=' . urlencode($stateID));

                SimpleSAML_Utilities::redirect($authorizeURL);
	}



	public function finalStep(&$state) {

		SimpleSAML_Logger::debug("oauth wrap:  Using this verification code [" .
			$state['authwindowslive:code'] . "]");

		$stateID = SimpleSAML_Auth_State::saveState($state, self::STAGE_INIT);
		// Retrieve Access Token
		// Documentation at: http://msdn.microsoft.com/en-us/library/ff749686.aspx
		$postData = 'client_id=' . $this->key
                               . '&redirect_uri=' . urlencode(SimpleSAML_Module::getModuleUrl('authwindowslive') . '/linkback.php?auth_state=' . urlencode($stateID))
                               . '&client_secret=' . urlencode($this->secret)
                               . '&code=' . urlencode($state['authwindowslive:code'])
                               . '&grant_type=authorization_code';


		$context = array(
			'http' => array(
				'method'  => 'POST',
				'header'  => 'Content-type: application/x-www-form-urlencoded',
				'content' => $postData,
			),
		);
		
		$result = SimpleSAML_Utilities::fetch('https://login.live.com/oauth20_token.srf', $context);
                $response = json_decode($result);

		// error checking of $response to make sure we can proceed
		if (empty($response->access_token))
                       throw new Exception('No access_token returned from Windows Live!');

		SimpleSAML_Logger::debug("Got an access token from the OAuth service provider [" .
                       $response->access_token . "]");

		$opts = array('http' => array('header' => "Accept: application/json"));

		$data = SimpleSAML_Utilities::fetch('https://apis.live.net/v5.0/me?access_token='.$response->access_token,$opts);
                $userdata = json_decode($data);

		$attributes = array();
              $attributes['windowslive_uid'] = array($userdata->id);
              $attributes['windowslive_targetedID'] = array('http://windowslive.com!' . $userdata->id);
              $attributes['windowslive_user'] = array($userdata->id . '@windowslive.com');

		if ($userdata->first_name)
                                $attributes['windowslive.FirstName'] = array($userdata->first_name);
                       if ($userdata->last_name)
                                $attributes['windowslive.LastName'] = array($userdata->last_name);
 
                       if ($userdata->emails->account)
                               $attributes['windowslive_mail'] = array($userdata->emails->account);




		SimpleSAML_Logger::debug('LiveID Returned Attributes: '. implode(", ",array_keys($attributes)));

		$state['Attributes'] = $attributes;
	}

}
